==> PyMSNt-0.9.5.tar.gz: PyMSNt-0.9.5/src/xdb.py <==
# Copyright 2004 James Bunton <james@delx.cjb.net>
# Licensed for distribution under the GPL version 2, check COPYING for details

import utils
if(utils.checkTwisted()):
	from twisted.xish.domish import Element
else:
	from tlib.domish import Element

import os
import os.path
import debug
import config
import legacy

SPOOL_UMASK = 0177

class XDB:
	"""
	Class for storage of data. Compatible with xdb_file from Jabberd1.4.x
	Allows PyMSN-t to be compatible with MSN-t
	
	Create one instance of the class for each XDB 'folder' you want.
	Call request()/set() with the xdbns argument you wish to retrieve
	"""
	def __init__(self, name, mangle=False):
		""" Creates an XDB object. If mangle is True then any '@' signs in filenames will be changed to '%' """
		self.name = utils.doPath(config.spooldir) + '/' + name
		if not os.path.exists(self.name) :
			os.makedirs(self.name)
		self.mangle = mangle
	
	def __getFile(self, file):
		if(self.mangle):
			file = file.replace('@', '%')
		
		document = utils.parseFile(self.name + "/" + file + ".xml")
		
		return document
	
	def __writeFile(self, file, text):
		if(self.mangle):
			file = file.replace('@', '%')
		
		prev_umask = os.umask(SPOOL_UMASK)
		f = open(self.name + "/" + file + ".xml", "w")
		f.write(text)
		f.close()
		os.umask(prev_umask)
	
	
	def request(self, file, xdbns):
		""" Requests a specific xdb namespace from the XDB 'file' """
		try:
			document = self.__getFile(file)
			for child in document.elements():
				if(child.getAttribute("xdbns") == xdbns):
					return child
		except:
			return None
	
	def set(self, file, xdbns, element):
		""" Sets a specific xdb namespace in the XDB 'file' to element """
		try:
			element.attributes["xdbns"] = xdbns
			document = None
			try:
				document = self.__getFile(file)
			except IOError:
				pass
			if(not document):
				document = Element((None, "xdb"))
			
			# Remove the existing node (if any)
			for child in document.elements():
				if(child.getAttribute("xdbns") == xdbns):
					document.children.remove(child)
			# Add the new one
			document.addChild(element)
			
			self.__writeFile(file, document.toXml())
		except:
			debug.log("XDB error writing entry %s to file %s" % (xdbns, file))
			raise
	
	def remove(self, file):
		""" Removes an XDB file """
		file = self.name + "/" + file + ".xml"
		if(self.mangle):
			file = file.replace('@', '%')
		try:
			os.remove(file)
		except:
			debug.log("XDB error removing file " + file)
			raise

==> PyMSNt-0.10-rc1.tar.gz: PyMSNt-0.10/src/xdb.py <==
# Copyright 2004-2005 James Bunton <james@delx.cjb.net>
# Licensed for distribution under the GPL version 2, check COPYING for details

import utils
if(utils.checkTwisted()):
	from twisted.xish.domish import Element
else:
	from tlib.domish import Element

import os
import os.path
import debug
import config
import legacy

SPOOL_UMASK = 0077

class XDB:
	"""
	Class for storage of data.
	
	Create one instance of the class for each XDB 'folder' you want.
	Call request()/set() with the xdbns argument you wish to retrieve
	"""
	def __init__(self, name, mangle=False):
		""" Creates an XDB object. If mangle is True then any '@' signs in filenames will be changed to '%' """
		self.name = utils.doPath(config.spooldir) + '/' + name
		if not os.path.exists(self.name):
			os.makedirs(self.name)
		self.mangle = mangle
	
	def __getFile(self, file):
		if(self.mangle):
			file = file.replace('@', '%')
		
		hash = file[0:2]
		document = utils.parseFile(self.name + "/" + hash + "/" + file + ".xml")
		
		return document
	
	def __writeFile(self, file, text):
		if(self.mangle):
			file = file.replace('@', '%')
		
		prev_umask = os.umask(SPOOL_UMASK)
		hash = file[0:2]
		pre = self.name + "/" + hash + "/"
		if not os.path.exists(pre):
			os.makedirs(pre)
		f = open(pre + file + ".xml", "w")
		f.write(text)
		f.close()
		os.umask(prev_umask)
	
	def files(self):
		""" Returns a list containing the files in the current XDB database """
		files = []
		for dir in os.listdir(self.name):
			if(os.path.isdir(self.name + "/" + dir)):
				files.extend(os.listdir(self.name + "/" + dir))
		if self.mangle:
			files = [x.replace('%', '@') for x in files]
		files = [x[:-4] for x in files]
		while files.count(''):
			files.remove('')

		return files
	
	def request(self, file, xdbns):
		""" Requests a specific xdb namespace from the XDB 'file' """
		try:
			document = self.__getFile(file)
			for child in document.elements():
				if(child.getAttribute("xdbns") == xdbns):
					return child
		except:
			return None
	
	def set(self, file, xdbns, element):
		""" Sets a specific xdb namespace in the XDB 'file' to element """
		try:
			element.attributes["xdbns"] = xdbns
			document = None
			try:
				document = self.__getFile(file)
			except IOError:
				pass
			if(not document):
				document = Element((None, "xdb"))
			
			# Remove the existing node (if any)
			for child in document.elements():
				if(child.getAttribute("xdbns") == xdbns):
					document.children.remove(child)
			# Add the new one
			document.addChild(element)
			
			self.__writeFile(file, document.toXml())
		except:
			debug.log("XDB error writing entry %s to file %s" % (xdbns, file))
			raise
	
	def remove(self, file):
		""" Removes an XDB file """
		file = self.name + "/" + file[0:2] + "/" + file + ".xml"
		if(self.mangle):
			file = file.replace('@', '%')
		try:
			os.remove(file)
		except:
			debug.log("XDB error removing file " + file)
			raise
